package main;

import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;

import javax.crypto.BadPaddingException;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;

import utils.Crypt;
import utils.Hash;
import utils.HashingException;

import ds.tree.*;


public class MeetInTheMiddle {
	
	RadixTree<String> haches;
	ArrayList<String> candidats;
	String chiffre;
	int taille;
	String mdp;
	
	
	
	public MeetInTheMiddle(String password) throws InvalidKeyException, DuplicateKeyException, NoSuchAlgorithmException, NoSuchPaddingException, IllegalBlockSizeException, BadPaddingException, InvalidAlgorithmParameterException, HashingException {
		taille=password.length()/2;
		mdp = password;
	}

	public boolean launch(boolean printInConsole) throws InvalidKeyException, DuplicateKeyException, NoSuchAlgorithmException, NoSuchPaddingException, IllegalBlockSizeException, BadPaddingException, InvalidAlgorithmParameterException, HashingException {
		long init=0, end=0;
		if(printInConsole)
			init = System.currentTimeMillis();
		testRadixTree(taille,printInConsole);
		if(printInConsole) {
			end = System.currentTimeMillis();
			System.out.println(mdp+":"+(end-init));
		}
		return candidats.size()>0;
	}

	//@SuppressWarnings("deprecation")
	private void testRadixTree(int lg, boolean printInConsole) throws InvalidKeyException, DuplicateKeyException, NoSuchAlgorithmException, NoSuchPaddingException, IllegalBlockSizeException, BadPaddingException, InvalidAlgorithmParameterException, HashingException {
		chiffre = Hash.hashDoubleDES(mdp,taille);
		if(printInConsole)
			System.out.println("* "+mdp+" -> "+Hash.hashDoubleDES(mdp,taille));
		haches = creerRadixTree();
		remplir("",lg);
		//((RadixTreeImpl<String>) haches).display();
		candidats = new ArrayList<String>();
		chercheCandidats("",lg);
		if(printInConsole) {
			for(int i=0;i<candidats.size();i++) {
				System.out.println("C "+candidats.get(i)+" -> "+Hash.hashDoubleDES(candidats.get(i),taille));
			}
		}
	}
	
	private void chercheCandidats(String string, int lg) throws InvalidKeyException, NoSuchAlgorithmException, NoSuchPaddingException, IllegalBlockSizeException, BadPaddingException, InvalidAlgorithmParameterException {
		if(lg>0) {
			for(int i=48;i<58;i++) {
				chercheCandidats(string+(char)i,lg-1);
			}
			for(int i=97;i<123;i++) {
				chercheCandidats(string+(char)i,lg-1);
			}
			for(int i=65;i<91;i++) {
				chercheCandidats(string+(char)i,lg-1);
			}
		} else {
			try {
				if(haches.contains(Crypt.decrypt(chiffre,string))) {
					candidats.add(haches.find(Crypt.decrypt(chiffre,string))+string);
				}
			} catch (Exception e) {
				// mauvais mot de passe
			}
		}
	}


	/* dans le radix tree, la cle est le crypte et la valeur est notre cle */
	private void remplir(String string, int lg) throws InvalidKeyException, DuplicateKeyException, NoSuchAlgorithmException, NoSuchPaddingException, IllegalBlockSizeException, BadPaddingException, InvalidAlgorithmParameterException, HashingException {
		if(lg>0) {
			for(int i=48;i<58;i++) {
				remplir(string+(char)i,lg-1);
			}
			for(int i=97;i<123;i++) {
				remplir(string+(char)i,lg-1);
			}
			for(int i=65;i<91;i++) {
				remplir(string+(char)i,lg-1);
			}
		} else {
			//System.out.println(string);
			if(!haches.contains(Hash.hash(string))) {
				haches.insert(Hash.hash(string),string);
			} else {
				System.out.println("j'ai trouve deux memes haches !");
			}
		}
	}

	public RadixTree<String> creerRadixTree() {
		return (new RadixTreeImpl<String>());
	}
	
}
