package test;

import static org.junit.Assert.*;

import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.Random;

import javax.crypto.BadPaddingException;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;

import org.junit.Test;

import utils.Crypt;

public class CryptTest {

	@Test
	public void enrypt() throws InvalidKeyException, NoSuchAlgorithmException, NoSuchPaddingException, IllegalBlockSizeException, BadPaddingException, InvalidAlgorithmParameterException {
		String iv = "12345678", key = "superKey";
		Random rand = new Random();
		
		for(int i=0;i<1000;++i){
			int ivp = rand.nextInt(), keyp = rand.nextInt();
			String encrypted = Crypt.encrypt(iv+ivp, key+keyp);
		
			assertFalse("encryptDES : encrypted message equals plain message", encrypted.equals(iv+ivp));
			assertFalse("encryptDES : encrypted message equals key", encrypted.equals(key+keyp));
			assertTrue("encryptDES : encrypted message is smaller than plain message", encrypted.length()*4 >= (iv+ivp).length()*8);
			assertTrue("encryptDES : encrypted message is not a bit set multiple of 64bits", (encrypted.length()*4)%64 == 0);
		}
	}
	
	@Test
	public void decrypt() throws InvalidKeyException, NoSuchAlgorithmException, NoSuchPaddingException, IllegalBlockSizeException, BadPaddingException, InvalidAlgorithmParameterException{
		String iv = "12345678", key = "superKey";
		Random rand = new Random();
		
		for(int i=0;i<1000;++i){
			int ivp = rand.nextInt(), keyp = rand.nextInt();
			String iv_t = iv+ivp, key_t = key+keyp;
			String encrypted = Crypt.encrypt(iv_t, key_t);
		
			assertTrue("decryptDES : has "+Crypt.decrypt(encrypted, key_t)+", "+iv_t+" expected", Crypt.decrypt(encrypted, key_t).equals(iv_t));
		}	
	}
	
	@Test
	public void hexToBytes(){
		String hexa = "FF0015C3";
		byte[] bytes = Crypt.hexToBytes(hexa);
		
		assertTrue("hexa->bytes : has "+bytes[0]+", -1 expected", bytes[0] == -1);
		assertTrue("hexa->bytes : has "+bytes[1]+", 0 expected", bytes[1] == 0);
		assertTrue("hexa->bytes : has "+bytes[2]+", 21 expected", bytes[2] == 21);
		assertTrue("hexa->bytes : has "+bytes[3]+", -61 expected", bytes[3] == -61);
	}
	
	@Test
	public void bytesToHex(){
		String hexa = "AB12CD9F36FF";
		byte[] bytes = Crypt.hexToBytes(hexa);
		
		assertTrue("bytes->hexa : has "+Crypt.bytesToHex(bytes)+", "+hexa+" expected", Crypt.bytesToHex(bytes).equalsIgnoreCase(hexa));
	}
}
