package utils;

public enum EncryptionMethod {
	// MethodName + Key length in bytes
    AES_CBC_NoPadding (new int[] {16}),
    AES_CBC_PKCS5Padding (new int[] {16}),
    AES_ECB_NoPadding (new int[] {16}),
    AES_ECB_PKCS5Padding (new int[] {16}),
    DES_CBC_NoPadding (new int[] {8}),
    DES_CBC_PKCS5Padding (new int[] {8}),
    DES_ECB_NoPadding (new int[] {8}),
    DES_ECB_PKCS5Padding (new int[] {8}),
    DESede_CBC_NoPadding (new int[] {21}),
    DESede_CBC_PKCS5Padding (new int[] {21}),
    DESede_ECB_NoPadding (new int[] {21}),
    DESede_ECB_PKCS5Padding (new int[] {21}),
    RSA_ECB_PKCS1Padding (new int[] {128, 256}),
    RSA_ECB_OAEPWithSHA1AndMGF1Padding (new int[] {128, 256}),
    RSA_ECB_OAEPWithSHA256AndMGF1Padding (new int[] {128, 256});

    private int[] lengths;
    private int max;
    private String mainAlgorithm;
    private String method;
    
    private EncryptionMethod(int[] lengths) {
    	this.lengths = (lengths == null || lengths.length < 0) ? new int[] {0} : lengths;
    	max = lengths[0];
    	for(int i : lengths)
    		if(i > max)
    			max = i;
    	
    	this.method = this.toString().replace('_', '/');
    	this.mainAlgorithm = this.method.substring(0,this.method.indexOf('/'));
    }
    
    public int[] getKeyLengths(){
    	return this.lengths;
    }
    
    public int getMaxKeyLength(){
    	return this.max;
    }
    
    public String getMainAlgorithm(){
    	return this.mainAlgorithm;
    }
    
    public String getMethod(){
    	return this.method;
    }
}
