package utils;

public class Hash {
	
	/**
	 * The IV used for every hash process.
	 */
	protected final static String IV = "0123456789";
	
	/**
	 * Computes the hash of the given password.
	 * @param password The password to hash.
	 * @return The hash of the password.
	 * @throws HashingException if an error occurred during hashing method.
	 */
	public static String hash(String password) throws HashingException{
		try {
			return Crypt.encrypt(IV, password);
		} catch (Exception e) {
			e.printStackTrace();
			throw new HashingException("Cannot hash '"+password+"' with IV '"+IV+"'.");
		}
	}
	
	public static String hashDoubleDES(String password,int n) throws HashingException{
		try {
			return Crypt.encrypt(Crypt.encrypt(IV, password.substring(0,n)), password.substring(n,2*n));
		} catch (Exception e) {
			e.printStackTrace();
			throw new HashingException("Cannot hash '"+password+"' with IV '"+IV+"'.");
		}
	}
	
	/**
	 * Computes the hash of the given password (size is limited to 4 characters).
	 * @param password The password to hash.
	 * @return The hash of the password.
	 * @throws HashingException if an error occurred during hashing method.
	 */
	public static String hash4(String password) throws HashingException{
		if(password.length() > 4)
			password = password.substring(0,4);
		return hash(password);
	}
	
	/**
	 * Computes the hash of the given password (size is limited to 6 characters).
	 * @param password The password to hash.
	 * @return The hash of the password.
	 * @throws HashingException if an error occurred during hashing method.
	 */
	public static String hash6(String password) throws HashingException{
		if(password.length() > 6)
			password = password.substring(0,6);
		return hash(password);
	}
	
	/**
	 * Computes the hash of the given password (size is limited to 8 characters).
	 * @param password The password to hash.
	 * @return The hash of the password.
	 * @throws HashingException if an error occurred during hashing method.
	 */
	public static String hash8(String password) throws HashingException{
		if(password.length() > 8)
			password = password.substring(0,8);
		return hash(password);
	}
}
